package doc;

class ToTextVisitor implements Visitor<String> {
    private final String prefix;
    private int sectionCtr = 0;
    private int imageCtr = 0;

    public ToTextVisitor() {
        this("");
    }

    public ToTextVisitor(String prefix) {
        this.prefix = prefix;
    }

    @Override
    public String visit(Paragraph section) {
        return section.getText();
    }

    @Override
    public String visit(Image image) {
        imageCtr++;
        final String num = prefix + imageCtr;
        return "<image " + image.getUrl() + ">\nFig. " + num + ": " + image.getCaption();
    }

    @Override
    public String visit(Section section) {
        final StringBuilder b = new StringBuilder();
        sectionCtr++;
        b.append(prefix).append(sectionCtr).append(" ").append(section.getHeader());
        final ToTextVisitor subVisitor = new ToTextVisitor(prefix + sectionCtr + ".");
        for (TextComponent c : section.getContents())
            b.append("\n\n").append(c.accept(subVisitor));
        return b.toString();
    }
}
